#include "cvsgroup.h"
#include "cvsgroup-internal.h"

#include <assert.h>
#include <errno.h>
#include <stdio.h>

static void
check_creating_groups (void)
{
    struct cvsgroup *devel;
    struct cvsgroup *mgmt;

    assert(create_cvsgroup(NULL) == NULL);
    assert(create_cvsgroup("group-no-percent") == NULL);

    devel = create_cvsgroup("%devel");
    assert(devel != NULL);
    assert(devel->groupname != NULL);
    assert(strcmp(devel->groupname, "%devel") == 0);
    assert(devel->users == NULL);
    assert(devel->users_count == 0);
    assert(devel->users_allocated == 0);

    assert(internal_add_user_to_group("alice", devel));
    assert(devel->users != NULL);
    assert(devel->users_count == 1);
    assert(devel->users_allocated >= devel->users_count + 1);
    assert(devel->users[0] != NULL);
    assert(strcmp(devel->users[0], "alice") == 0);
    assert(devel->users[1] == NULL);

    assert(internal_add_user_to_group("bob", devel));
    assert(devel->users != NULL);
    assert(devel->users_count == 2);
    assert(devel->users_allocated >= devel->users_count + 1);
    assert(devel->users[0] != NULL);
    assert(strcmp(devel->users[0], "alice") == 0);
    assert(devel->users[1] != NULL);
    assert(strcmp(devel->users[1], "bob") == 0);
    assert(devel->users[2] == NULL);

    mgmt = create_cvsgroup("%mgmt");
    assert(mgmt != NULL);

    devel = get_cvsgroup("%devel");
    assert(devel != NULL);
    assert(strcmp(devel->groupname, "%devel") == 0);

    /* no duplicates allowed */
    assert(create_cvsgroup("%devel") == NULL); 

    clear_groups();
}

static void
check_adding_users (void)
{

    assert(!add_user_to_group("alice", "group-no-percent"));

    assert(add_user_to_group("alice", "%devel"));

    assert(user_in_group("alice", "%devel"));

    assert(!user_in_group("carol", "%devel"));

    assert(!user_in_group("alice", "%mngnt"));

    clear_groups();
}


static void
check_reading_groups_file (void)
{
    assert(read_cvsgroup_file("./test-groups/cvsgroup") == 0);

/*    assert(user_in_group("alice", "%devel"));

      assert(!user_in_group("carol", "%devel"));*/

    clear_groups();
}

int
main (void)
{
    check_creating_groups(); 

    check_adding_users();

    assert((read_cvsgroup_file("./test-groups/cvsgroup-nonexisting") == -1 &&
	    (errno == ENOENT)));

    assert(read_cvsgroup_file("./test-groups/cvsgroup-too-long") == 1);

    assert(read_cvsgroup_file("./test-groups/cvsgroup-broken1") == 1);

    assert(read_cvsgroup_file("./test-groups/cvsgroup-ignore") == 0);
    
    check_reading_groups_file();

    exit(0);
}
